@@ -85,13 +85,14 @@ sub choose_subquery {
     my $old_idx = 1;
 
     SUBQUERY: while ( 1 ) {
-        my $info = $ax->get_sql_info( $sql );
         my $menu = [ @pre, map( '- ' . $_->[1], @$history_HD ), map( '  ' . $_->[1], @$history_RAM ) ];
+        my $info = $ax->get_sql_info( $sql );
         # Choose
         my $idx = $tc->choose(
             $menu,
             { %{$sf->{i}{lyt_v}}, info => $info, prompt => '', index => 1, default => $old_idx, undef => '  <=' }
         );
+        $ax->print_sql_info( $info );
         if ( ! defined $idx || ! defined $menu->[$idx] ) {
             return;
         }
@@ -107,10 +108,8 @@ sub choose_subquery {
                 ( $history_HD, $history_RAM ) = $sf->__get_history();
                 $menu = [ @pre, map( '- ' . $_->[1], @$history_HD ), map( '  ' . $_->[1], @$history_RAM ) ];
             }
-            $ax->print_sql_info( $sql );
             next SUBQUERY;
         }
-        $info = $ax->get_sql_info( $sql );
         my ( $prompt, $default );
         if ( $menu->[$idx] eq $readline ) {
             $prompt = 'Enter SQ: ';
@@ -120,14 +119,15 @@ sub choose_subquery {
             $idx -= @pre;
             $default = ( @$history_HD, @$history_RAM )[$idx][0];
         }
+        $info = $ax->get_sql_info( $sql );
         my $tf = Term::Form->new( $sf->{i}{tf_default} );
         # Readline
         my $stmt = $tf->readline(
             $prompt,
             { default => $default, show_context => 1, info => $info }
         );
+        $ax->print_sql_info( $info );
         if ( ! defined $stmt || ! length $stmt ) {
-            $ax->print_sql_info( $sql );
             next SUBQUERY;
         }
         my $db = $sf->{d}{db};
@@ -159,11 +159,13 @@ sub __edit_sq_file {
             map( line_fold( $_->[-1], get_term_width(), { init_tab => '  ', subseq_tab => '    ', join => 1 } ), @$history_HD ), #
             ' '
         );
+        my $info = join( "\n", @info );
         # Choose
         my $choice = $tc->choose(
             [ @pre, $add, $edit, $remove ],
-            { %{$sf->{i}{lyt_v}}, info => join( "\n", @info ), undef => '  <=' }
+            { %{$sf->{i}{lyt_v}}, info => $info, undef => '  <=' }
         );
+        $ax->print_sql_info( $info );
         my $changed = 0;
         if ( ! defined $choice ) {
             return $any_change;
@@ -193,6 +195,7 @@ sub __edit_sq_file {
 
 sub __add_subqueries {
     my ( $sf, $history_HD, $top_lines ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $history_RAM = $sf->__tmp_history( $history_HD );
@@ -212,11 +215,13 @@ sub __add_subqueries {
         }
         push @info, ' ';
         my $menu = [ @pre, map {  '- ' . $_->[1] } @$history_RAM ];
+        my $info = join( "\n", @info );
         # Choose
         my $idx = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_v}}, prompt => 'Add:', info => join( "\n", @info ), index => 1 }
+            { %{$sf->{i}{lyt_v}}, prompt => 'Add:', info => $info, index => 1 }
         );
+        $ax->print_sql_info( $info );
         if ( ! $idx ) {
             if ( @$bu ) {
                 ( $added_sq, $history_RAM, $used ) = @{pop @$bu};
@@ -229,11 +234,13 @@ sub __add_subqueries {
             return 1;
         }
         elsif ( $menu->[$idx] eq $readline ) {
+            my $info = join( "\n", @info );
             # Readline
             my $stmt = $tf->readline(
                 'Stmt: ',
-                { info => join( "\n", @info ), show_context => 1, clear_screen => 1 }
+                { info => $info, show_context => 1, clear_screen => 1 }
             );
+            $ax->print_sql_info( $info );
             if ( ! defined $stmt || ! length $stmt ) {
                     next;
             }
@@ -241,11 +248,13 @@ sub __add_subqueries {
                 $stmt = $1;
             }
             my $folded_stmt = "\n" . line_fold( 'Stmt: ' . $stmt, get_term_width(), { init_tab => '', subseq_tab => ' ' x length( 'Stmt: ' ), join => 1 } );
+            $info = join( "\n", @info ) . $folded_stmt;
             # Readline
             my $name = $tf->readline(
                 'Name: ',
-                { info => join( "\n", @info ) . $folded_stmt, show_context => 1 }
+                { info => $info, show_context => 1 }
             );
+            $ax->print_sql_info( $info );
             if ( ! defined $name ) {
                 next;
             }
@@ -258,11 +267,13 @@ sub __add_subqueries {
             push @$used, splice @$history_RAM, $idx-@pre, 1;
             my $stmt = $used->[-1][0];
             my $folded_stmt = "\n" . line_fold( 'Stmt: ' . $stmt, get_term_width(), { init_tab => '', subseq_tab => ' ' x length( 'Stmt: ' ), join => 1 } );
+            my $info = join( "\n", @info ) . $folded_stmt;
             # Readline
             my $name = $tf->readline(
                 'Name: ',
-                { info => join( "\n", @info ) . $folded_stmt, show_context => 1 }
+                { info => $info, show_context => 1 }
             );
+            $ax->print_sql_info( $info );
             if ( ! defined $name ) {
                 ( $added_sq, $history_RAM, $used ) = @{pop @$bu};
                 next;
@@ -278,6 +289,7 @@ sub __edit_subqueries {
     if ( ! @$history_HD ) {
         return;
     }
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $indexes = [];
     my @pre = ( undef, $sf->{i}{_confirm} );
@@ -286,18 +298,19 @@ sub __edit_subqueries {
     my @unchanged_saved_history = @$history_HD;
 
     STMT: while ( 1 ) {
-        my $info = join "\n", @$top_lines;
         my @available;
         for my $i ( 0 .. $#$history_HD ) {
             my $pre = ( any { $i == $_ } @$indexes ) ? '| ' : '- ';
             push @available, $pre . $history_HD->[$i][1];
         }
         my $menu = [ @pre, @available ];
+        my $info = join "\n", @$top_lines;
         # Choose
         my $idx = $tc->choose(
             $menu,
             { %{$sf->{i}{lyt_v}}, prompt => 'Edit:', info => $info, index => 1, default => $old_idx }
         );
+        $ax->print_sql_info( $info );
         if ( ! $idx ) {
             if ( @$bu ) {
                 ( $history_HD, $indexes ) = @{pop @$bu};
@@ -338,6 +351,7 @@ sub __edit_subqueries {
                 'Stmt: ',
                 { info => $info, default => $history_HD->[$idx][0], show_context => 1, clear_screen => 1 }
             );
+            $ax->print_sql_info( $info );
             if ( ! defined $stmt || ! length $stmt ) {
                 next STMT;
             }
@@ -346,11 +360,13 @@ sub __edit_subqueries {
             if ( $history_HD->[$idx][0] ne $history_HD->[$idx][1] ) {
                 $default = $history_HD->[$idx][1];
             }
+            $info .= $folded_stmt;
             # Readline
             my $name = $tf->readline(
                 'Name: ',
-                { info => $info . $folded_stmt, default => $default, show_context => 1 }
+                { info => $info, default => $default, show_context => 1 }
             );
+            $ax->print_sql_info( $info );
             if ( ! defined $name ) {
                 next STMT;
             }
@@ -369,17 +385,18 @@ sub __remove_subqueries {
     if ( ! @$history_HD ) {
         return;
     }
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tu = Term::Choose::Util->new( $sf->{i}{tcu_default} );
+    my $info = join( "\n", @$top_lines );
     my $idxs = $tu->choose_a_subset(
         [ map { $_->[1] } @$history_HD ],
         {
-            info => join( "\n", @$top_lines ),
-            cs_label => "Remove:\n", cs_begin => "  ", cs_separator => "\n  ", cs_end => "\n",
-            prompt => 'Choose:', back => $sf->{i}{_back}, confirm => $sf->{i}{_confirm},
-            prefix => '- ', layout => 3, clear_screen => 1,
-            index => 1, all_by_default => 0
+            info => $info, cs_label => "Remove:\n", cs_begin => "  ", cs_separator => "\n  ", cs_end => "\n",
+            prompt => 'Choose:', back => $sf->{i}{_back}, confirm => $sf->{i}{_confirm}, prefix => '- ', layout => 3,
+            clear_screen => 1, index => 1, all_by_default => 0
         }
     );
+    $ax->print_sql_info( $info );
     if ( ! defined $idxs || ! @$idxs ) {
         return;
     }
